@@ -257,7 +257,7 @@ DEPRECATED_PATTERN = re.compile(r"^\*DEPRECATED(?P<message>.*)\*(?P<doc>.*)")
 
 @dataclass
 class ArgumentSpec:
-    name: str
+    name: Optional[str]
     type: str
     positional_only: List[str]
     positional_or_named: List[str]
@@ -377,6 +377,17 @@ class KeywordDoc(SourceEntity):
         else:
             return Range.invalid()
 
+    @single_call
+    def normalized_tags(self) -> List[str]:
+        return [normalize(tag) for tag in self.tags]
+
+    @single_call
+    def is_private(self) -> bool:
+        if get_robot_version() < (6, 0, 0):
+            return False
+
+        return "robot:private" in self.normalized_tags()
+
     @property
     def range(self) -> Range:
         if self.name_token is not None:
@@ -448,6 +459,13 @@ class KeywordDoc(SourceEntity):
                     f"| {' or '.join(f'`<{s}>`' for s in a.types) if a.types is not None else ''} |"
                 )
 
+        if self.tags:
+            if result:
+                result += "\n\n"
+
+            result += "**Tags**: \n- "
+            result += "\n- ".join(self.tags)
+
         return result
 
     @property
